/* Config.hh. Generated by CMake for ignition-sensors. */

/* Version number */
#define IGNITION_SENSORS_MAJOR_VERSION 6
#define IGNITION_SENSORS_MINOR_VERSION 4
#define IGNITION_SENSORS_PATCH_VERSION 0

#define IGNITION_SENSORS_VERSION "6.4"
#define IGNITION_SENSORS_VERSION_FULL "6.4.0"

#define IGNITION_SENSORS_VERSION_NAMESPACE v6

#define IGNITION_SENSORS_VERSION_HEADER "Ignition Sensors, version 6.4.0\nCopyright (C) 2014 Open Source Robotics Foundation.\nReleased under the Apache 2.0 License.\n\n"

/* #undef BUILD_TYPE_PROFILE */
/* #undef BUILD_TYPE_DEBUG */
#define BUILD_TYPE_RELEASE 1

#define IGN_SENSORS_PLUGIN_PATH "D:/bld/libignition-sensors6_1652503203651/_h_env/Library/lib"
#define IGN_SENSORS_PLUGIN_NAME(name) std::string("ignition-sensors6-")+name
