/*
 * Copyright (C) 2017 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

//                ****** Do not modify this file. ******
// This file is automatically generated by CMake. Changes should instead be
// made to cmake/ign_auto_headers.hh.in in ignition-cmake

#include <ignition/sensors/config.hh>
#include <ignition/sensors/AirPressureSensor.hh>
#include <ignition/sensors/AltimeterSensor.hh>
#include <ignition/sensors/BrownDistortionModel.hh>
#include <ignition/sensors/CameraSensor.hh>
#include <ignition/sensors/DepthCameraSensor.hh>
#include <ignition/sensors/Distortion.hh>
#include <ignition/sensors/ForceTorqueSensor.hh>
#include <ignition/sensors/GaussianNoiseModel.hh>
#include <ignition/sensors/GpuLidarSensor.hh>
#include <ignition/sensors/ImageBrownDistortionModel.hh>
#include <ignition/sensors/ImageDistortion.hh>
#include <ignition/sensors/ImageGaussianNoiseModel.hh>
#include <ignition/sensors/ImageNoise.hh>
#include <ignition/sensors/ImuSensor.hh>
#include <ignition/sensors/Lidar.hh>
#include <ignition/sensors/LogicalCameraSensor.hh>
#include <ignition/sensors/MagnetometerSensor.hh>
#include <ignition/sensors/Manager.hh>
#include <ignition/sensors/NavSatSensor.hh>
#include <ignition/sensors/Noise.hh>
#include <ignition/sensors/RenderingEvents.hh>
#include <ignition/sensors/RenderingSensor.hh>
#include <ignition/sensors/RgbdCameraSensor.hh>
#include <ignition/sensors/SegmentationCameraSensor.hh>
#include <ignition/sensors/Sensor.hh>
#include <ignition/sensors/SensorFactory.hh>
#include <ignition/sensors/SensorTypes.hh>
#include <ignition/sensors/ThermalCameraSensor.hh>
#include <ignition/sensors/Util.hh>

