



if not exist %PREFIX%\\Library\\include\\ignition\\sensors6\\ignition\\sensors.hh exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\ignition-sensors6.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\ignition-sensors6.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\cmake\\ignition-sensors6\\ignition-sensors6-config.cmake exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
